;
; OutRun PETSCII
;
; a tribute to Shine/G*P
;
; released by Arkanix Labs in 2020
;
; www.arkanixlabs.com
;
; code and graphics by
; aNdy (CSDb id=6404)
;
; music (from C64 game) by
; jason c. brooke (CSDb id=8136)
;
; testing by
; moloch (CSDb id=930)
;
; This source is formatted for the ACME cross assembler
; which can be downloaded from...
; http://sourceforge.net/projects/acme-crossass/
;
; The final build was compressed and squished using
; Exomizer 2 which can be downloaded from...
; http://hem.bredband.net/magli143/exo/
;
; Questions about this code or about how the binaries
; were created can be asked by emailing aNdy...
;
; andy@cosine.org.uk
;

; the name and file format for the assembled program

		!to "outrun-petscii.prg",cbm

; claw in the binary data...
		
		* = $2000
		!binary "data/font.chr"
		
		* = $b000
music	!binary "data\music.prg",,2

; label assignments for keyboard control

cpra				= $dc00				; cia #1, port register a
cprb				= $dc01				; cia #1, port register b

; some more label assignments (variables)

sync				= $50				; interupt sync variable
video_type			= $51				; 0=pal, 1=ntsc
music_tmr			= $52				; timer for music frame skip on ntsc		


; add a BASIC startline - generate sys 2066

		* = $0801
		!word entry-2
		!byte $00,$00,$9e
		!text "2066"
		!byte $00,$00,$00


; entry point at $0812 (sys 2066!)

		* = $0812

entry

; detect what machine type is being used...

start_scan
		lda $d012
chkntsc
		cmp $d012
		beq chkntsc
		bmi start_scan
		adc #$00
		lsr
		bcs isntsc
		lsr
		bcs isoldntsc
		ldx #$00					; check drean or pal
		lda #$10
chkpal
		inx
		cmp $d012
		bne chkpal
		cpx #$70
		bcc ispal
isdrean
		lda #$01
		sta video_type				; video = ntsc
		jmp detect_dun
isntsc	
		lda #$01
		sta video_type				; video = ntsc
		jmp detect_dun
isoldntsc
		lda #$01
		sta video_type				; video = ntsc
		jmp detect_dun
ispal
		lda #$00
		sta video_type				; video = pal

detect_dun
		lda #$35					; kill EVERYTHING (MWHAHA!) except $f000-$ffff
		sta $01

; stop interrupts, disable the roms and set up nmi and irq interrupt pointers	

		sei							; disable maskable IRQs		
		
; setup nmi

		lda #<nmi
		sta $fffa
		lda #>nmi
		sta $fffb
		
; setup irq's

		lda #<int
		sta $fffe
		lda #>int
		sta $ffff

		lda #$7f
		sta $dc0d			; disable timer interrupts which can be generated by the two CIA chips
		sta $dd0d			; the kernal uses such an interrupt to flash the cursor and scan the keyboard, so we better stop it

		lda $dc0d			; by reading this two registers we negate any pending CIA irqs.
		lda $dd0d			; if we don't do this, a pending CIA irq might occur after we finish setting up our irq. we don't want that to happen.

		lda #$00			; where raster starts on screen, usually at top but can be different
		sta $d012

		lda #$1b			; as there are more than 256 rasterlines, the topmost bit of $d011 serves as
		sta $d011			; the 9th bit for the rasterline we want our irq to be triggered.
		
		lda #$01
		sta $d019			; clear interupt bit
		sta $d01a			; tell vicii to generate interupt

		lda #$35			; turn off basic and kernal rom thus
		sta $01				; c64 now sees RAM everywhere except at $d000-$e000
		
		lda #$03			; select video bank
		sta $dd00		
						
		lda #$18			; use our custom charset (lowecase rom)
		sta $d018

		cli
		
; prepare the music	

		lda #$01
		jsr music+$219		
		
; setup the screen for the very first run

		jsr clear_screen			; clear screen with subroutine

; print intro message		
				
		ldx #$00
text_loop
		lda intro_1,x				; pull in the data for the
		sta $0590,x					; 3 line 'intro' text
		lda intro_2,x				; and print it on screen!
		sta $05e0,x		
		lda intro_3,x
		sta $0630,x			
	
		lda #$01					; colour those lines!
		sta $d990,x
		lda #$0f		
		sta $d9e0,x
		lda #$0c		
		sta $da30,x		

		inx
		cpx #$28
		bne text_loop

; give time to read the intro	
						
		ldy #$ff					; wait a bit...
		jsr sync_wait_long	

		jsr clear_screen			; clear screen with subroutine
		
; print up the main screen ram			
		
		ldx #$00					; zero x register
scrnram_loop						; load up whatver is in the screen
		lda screen_ram,x			; ram table and write to the
		sta $0400,x					; screen ram
		lda screen_ram+250,x
		sta $0400+250,x
		lda screen_ram+500,x
		sta $0400+500,x
		lda screen_ram+750,x
		sta $0400+750,x	
		inx
		cpx #$fa					; done 250 writes?
		bne scrnram_loop			; all written? no? loop back!
		
; print the compressed title screen colour data to colour ram
			
		ldx #$00					; zero x register
		ldy #$00					; zero y register
colram_loop
		lda colour_ram,x			; load up a byte
		sta $d800+1,y				; store a nybble to screen
		lsr
		lsr
		lsr
		lsr
		sta $d800,y					; store the other nybble

		lda colour_ram+125,x
		sta $d800+251,y
		lsr
		lsr
		lsr
		lsr
		sta $d800+250,y

		lda colour_ram+250,x
		sta $d800+501,y
		lsr
		lsr
		lsr
		lsr
		sta $d800+500,y

		lda colour_ram+375,x
		sta $d800+751,y
		lsr
		lsr
		lsr
		lsr
		sta $d800+750,y

		iny
		iny
		inx
		cpx #$7d
		bne colram_loop			
		
; main loop to detect keypresses

mainloop

		jsr sync_wait				; wait until irq finishes!

; check if 's' is pressed for 'splash wave' music

		lda #$fd 					; load $dc00...
		sta cpra					; ...with bit row 1
		
		lda cprb					; load $dc01 to test...
		cmp #$df					; key 's' pressed?
		bne check_m					; no? check 'm'
		
		jsr key_debounce			; debounce the keyboard!
		
		lda #$01
		jsr music+$219
		
		jmp kcheck_dun		
		

; check if 'm' is pressed for 'magical sound shower' music

check_m	
		lda #$ef 					; load $dc00...
		sta cpra					; ...with bit row 4
		
		lda cprb					; load $dc01 to test...
		cmp #$ef					; key 'm' pressed?
		bne check_space
		
		jsr key_debounce
		
		lda #$00
		jsr music+$219
		
		jmp kcheck_dun	
		
; check if 'm' is pressed for 'magical sound shower' music	

check_space
		lda #$7f 					; load $dc00...
		sta cpra					; ...with bit row 7
		
		lda cprb					; load $dc01 to test...
		cmp #$ef					; key 'm' pressed?
		bne kcheck_dun
		
		jsr key_debounce
		jmp exit_demo
		
kcheck_dun			
		jmp mainloop
	
; space has been pressed so reset some registers

exit_demo
		sei
		lda #$37
		sta $01

		lda #$00
		sta $d011
		sta $d020
		sta $d021
		sta $d418

; reset the C64

		jmp $fce2			
												

; irq interrupt =================================

int
		pha  
		txa
		pha
		tya
		pha

		lda $d019
		and #$01
		sta $d019

; play some music!

		lda video_type				; what machine is being used
		cmp #$00					; PAL?
		beq call_music_driver		; yes? play the music!
									; no?
		ldx music_tmr				; load music timer
		inx							; increase it
		cpx #$08					; is it 8?
		bne	music_play				; no? play music!
									; yes?
		ldx #$00					; reset the timer
		stx music_tmr				; then skip playing
									; the music on this occasion
		jmp set_sync				; to slow it for ntsc!

music_play
		stx	music_tmr
		
call_music_driver
		jsr music+$69

; set sync variable for main line code synchronisation

set_sync
		lda #$01					; set the sync flag to '1'		
		sta sync					; so the main code runs!
		
		pla
		tay
		pla
		tax
		pla
nmi		rti

; end irq interrupt =============================


; subroutines ===================================


; clear the screen!

clear_screen
        lda #$00
		sta $d020
		sta $d021
		tax
		lda #$20
clsloop
		sta $0400,x
		sta $0500,x
		sta $0600,x
		sta $0700,x
		dex
		bne clsloop

		rts

; debounce subroutines for keyboard -------------------

key_debounce		
		lda cprb						; is there a keypress still?
		cmp #$ff						; an idiot is still pressing something???
		bne key_debounce				; lock them in a loop till they stop!!
		
		rts	

; irq sync wait subroutine ---------------------------------- 
	   	 
sync_wait								; wait for sync update in irq code!
		lda #$00						; waits until the sync flag is
		sta sync						; set at the end of the interupt
sw_loop									; code further down.
		cmp sync						; no code in the main loop above
		beq sw_loop						; is executed until the interupt
										; has run it's course!
		rts

; a sync wait delay subroutine ---------------------------------- 		
		
sync_wait_long						
		jsr sync_wait
		dey
		bne sync_wait_long

		rts							
			   	
; end subroutines ===============================


; screen data here! ==========================


; text data for intro message		
		
intro_1		!scr " Arkanix Labs presents 'OutRun PETSCII' "
intro_2		!scr "     A small tribute to Shine / G*P     "
intro_3		!scr " Code/GFX: aNdy  Music: Jason C. Brooke "


; main screen ram data

screen_ram
!byte $A0,$A0,$E4,$E4,$E4,$E4,$A0,$A0,$A0,$A0,$A0,$A0,$E4,$E4,$E4,$E4,$E4,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$E4,$E4,$E4,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$EA,$D4,$C9,$CD,$C5,$F4,$A0,$35,$38,$A0,$EA,$D3,$C3,$CF,$D2,$C5,$F4,$B0,$B0,$B0,$B0,$B0,$B0,$B0,$B0,$EA,$CC,$C1,$D0,$F4,$B0,$A7,$B1,$B7,$A2,$B2,$B9,$A0,$A0
!byte $A0,$A0,$E3,$E3,$E3,$E3,$A0,$A0,$A0,$A0,$A0,$A0,$E3,$E3,$E3,$E3,$E3,$A0,$E3,$E3,$E3,$E3,$E3,$E3,$E3,$E3,$A0,$E3,$E3,$E3,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$EC,$E2,$E2,$FB,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $EC,$FF,$A2,$A0,$7F,$FB,$FB,$A0,$EC,$E2,$FB,$A0,$A0,$A0,$A0,$A0,$F0,$EE,$E8,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $FE,$A7,$A0,$A7,$A2,$FC,$FC,$F9,$FF,$A2,$7F,$FB,$A0,$A0,$A0,$A0,$ED,$FD,$62,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $FE,$AE,$A2,$7F,$A0,$EC,$7F,$A0,$A0,$7F,$A0,$A2,$FF,$EF,$A0,$A0,$61,$DC,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $AE,$E8,$BA,$A5,$BB,$A0,$BA,$A0,$BA,$A0,$AC,$7F,$FE,$A7,$FF,$A0,$F0,$EE,$E8,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $E6,$BA,$A3,$E9,$A0,$BA,$A0,$EC,$AE,$BA,$BA,$AC,$A0,$BA,$A2,$FB,$ED,$FD,$62,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $FF,$E6,$E9,$EC,$E8,$E8,$E8,$BA,$A3,$E6,$E8,$FE,$A5,$E8,$E8,$FC,$E8,$E8,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4,$E4
!byte $A0,$5C,$A0,$A0,$A0,$A0,$A0,$DC,$DE,$69,$A0,$DC,$E9,$A0,$A0,$AA,$AE,$BA,$A7,$A0,$A2,$BA,$AE,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$5C,$A0,$A0,$A0,$A0,$A0,$A0,$5C,$A0,$A0,$A0,$F5,$AE,$BA,$A2,$A0,$AE,$BA,$A0,$BA,$AE,$A0,$A2,$BA,$AE,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$5C,$A0,$A0,$A0,$A0,$A0,$A0,$5C,$A0,$AE,$BA,$A2,$A0,$A0,$A0,$BA,$A0,$A0,$A0,$A0,$A0,$BA,$A0,$A0,$A0,$A2,$BA,$AE,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$5C,$A0,$A0,$A0,$A0,$A0,$AE,$BA,$A2,$A0,$A0,$A0,$A0,$A0,$AE,$E4,$EF,$EF,$A0,$EF,$EF,$E4,$AE,$A0,$A0,$A0,$A0,$A0,$A2,$BA,$AE,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$5C,$A0,$A0,$AE,$BA,$A2,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$BA,$EC,$BC,$9B,$9D,$F7,$A8,$A9,$BE,$FB,$BA,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A2,$BA,$AE,$A0,$A0,$A0,$A0,$A0
!byte $A0,$AE,$BA,$A2,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$AE,$A0,$A8,$E6,$E8,$E8,$E8,$E8,$E8,$E6,$A9,$A0,$AE,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A2,$BA,$AE,$A0,$A0
!byte $A2,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$BA,$A0,$A0,$0F,$0F,$64,$64,$25,$64,$64,$0F,$0F,$A0,$A0,$BA,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A2,$BA
!byte $A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$AE,$A0,$A0,$A0,$DF,$9D,$AA,$D3,$C8,$CE,$AA,$9B,$E9,$A0,$A0,$A0,$AE,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$BA,$A0,$A0,$A0,$A0,$62,$62,$F7,$E3,$E3,$E3,$F7,$62,$62,$A0,$A0,$A0,$A0,$BA,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$81,$CE,$84,$99
!byte $A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
!byte $A0,$30,$30,$30,$A0,$0B,$0D,$2F,$08,$A0,$31,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$53,$54,$41,$47,$45,$A0,$31,$A0,$A0,$A0
!byte $A0,$75,$75,$75,$75,$75,$75,$75,$75,$75,$75,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0,$A0
		
; main colour ram data

colour_ram
!byte $EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE
!byte $EE,$22,$22,$EE,$77,$EE,$44,$44,$4E,$33,$33,$33,$33,$E6,$66,$EE,$EE,$EE,$EE,$EE
!byte $EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE
!byte $EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE
!byte $EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE
!byte $EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE
!byte $EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE
!byte $E1,$11,$1E,$EE,$EE,$EE,$EE,$EE,$D5,$E1,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
!byte $11,$11,$11,$1E,$11,$1E,$EE,$EE,$55,$E0,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10
!byte $55,$51,$11,$11,$11,$11,$EE,$EE,$EE,$E1,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
!byte $55,$55,$11,$11,$11,$11,$11,$EE,$D5,$E0,$10,$10,$10,$10,$10,$10,$10,$10,$10,$10
!byte $55,$55,$11,$11,$55,$11,$11,$1E,$55,$E1,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
!byte $55,$55,$11,$15,$55,$15,$51,$11,$1E,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EE
!byte $C8,$CC,$CC,$C5,$55,$C5,$5C,$C5,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC
!byte $F8,$FF,$FF,$FF,$8F,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
!byte $C8,$CC,$CC,$CC,$8C,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC
!byte $F8,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
!byte $C8,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$2A,$AC,$77,$2C,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC
!byte $FF,$FF,$FF,$FF,$FF,$FF,$FF,$F2,$22,$22,$22,$22,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
!byte $CC,$CC,$CC,$CC,$CC,$CC,$CC,$C2,$A2,$2A,$22,$A2,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC
!byte $FF,$FF,$FF,$FF,$FF,$FF,$FF,$F2,$22,$AA,$A2,$22,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
!byte $CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC,$CC
!byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
!byte $02,$22,$07,$77,$70,$50,$00,$00,$00,$00,$00,$00,$00,$00,$00,$77,$77,$70,$70,$00
!byte $03,$33,$35,$55,$44,$40,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

